import { InternalProvider } from './InternalProvider';
import { Factory, ILifetime } from './ILifetime';
import { Keys } from './NameSelector';
declare type LifetimeProvider<T, E> = new (container: Container<E>, className: string, providerName: string, factoryFunction: Factory<T, E>) => ILifetime<T, E>;
declare type NameSelector<E> = (provider: Keys<E>) => string;
declare type ProviderProvider<E> = {
    prototype: E;
    name: string;
    new (): E;
};
declare type DependencyProvider<T, E> = {
    prototype: T;
    name: string;
    new (provider: E): T;
} | {
    prototype: T;
    name: string;
    new (): T;
};
declare type DependencyOptions<T, E> = {
    factory?: Factory<T, E>;
    name?: NameSelector<E>;
};
export declare type DependencyGetter<T, E> = DependencyProvider<T, E> | string;
declare type ActualProvider<E> = E & InternalProvider<E>;
export declare class Container<E> {
    readonly template: E;
    private readonly dependencyLookup;
    private readonly providerLookup;
    private readonly intendedDependencies;
    private provider?;
    constructor(ProviderTemplate: ProviderProvider<E>);
    addSingleton<T>(Dependency: DependencyProvider<T, E>, options?: DependencyOptions<T, E>): void;
    tryAddSingleton<T>(Dependency: DependencyProvider<T, E>, options?: DependencyOptions<T, E>): void;
    addTransient<T>(Dependency: DependencyProvider<T, E>, options?: DependencyOptions<T, E>): void;
    tryAddTransient<T>(Dependency: DependencyProvider<T, E>, options?: DependencyOptions<T, E>): void;
    add<T>(Lifetime: LifetimeProvider<T, E>, Dependency: DependencyProvider<T, E>, options?: DependencyOptions<T, E>): void;
    tryAdd<T>(Lifetime: LifetimeProvider<T, E>, Dependency: DependencyProvider<T, E>, { factory, name }?: DependencyOptions<T, E>): boolean;
    /**
     * Calling this there is no guarantee that it can provide an instance
     */
    get<T>(item: DependencyGetter<T, E>): ILifetime<T, E> | undefined;
    preBuildValidate(): void;
    build(): ActualProvider<E>;
}
export {};
